/*------------------------------------------------------------------------
  File  : rtb/jira/IssueAction.cls
  Desc  : Jira Issue
  Notes : Manipulate a Jira Issue
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING OpenEdge.Net.HTTP.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.IssueAction INHERITS JiraData:
  
  DEFINE PRIVATE PROPERTY issueKey AS CHARACTER GET. SET. 

  CONSTRUCTOR IssueAction (INPUT pcIssueKey AS CHARACTER):
    SUPER().
    THIS-OBJECT:issueKey = pcIssueKey.
  END CONSTRUCTOR.
  
  METHOD PUBLIC VOID addComment (INPUT pcComment AS CHARACTER):
    
    DEFINE VARIABLE cUrl           AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE commentPayload AS JsonObject NO-UNDO.

    commentPayload = NEW JsonObject().
    commentPayload:Add("body", pcComment).
    
    cUrl = jiraConfig:jiraUrl + "issue/" + THIS-OBJECT:issueKey + "/" + "comment".
    
    postRequest (cUrl, commentPayload).  
      
    FINALLY:
      DELETE OBJECT commentPayload NO-ERROR.
    END FINALLY.
    
  END METHOD.
  
  METHOD PUBLIC VOID applyTransition (INPUT pcTransitionId AS CHARACTER):
    
    DEFINE VARIABLE cUrl              AS CHARACTER  NO-UNDO.
    DEFINE VARIABLE transitionPayload AS JsonObject NO-UNDO.
    DEFINE VARIABLE transitionFields  AS JsonObject NO-UNDO.

    ASSIGN 
      transitionPayload = NEW JsonObject()
      transitionFields  = NEW JsonObject().
   
    transitionFields:Add("id", pcTransitionId).
    transitionPayload:Add("transition", transitionFields).
    
    cUrl = jiraConfig:jiraUrl + "issue/" + THIS-OBJECT:issueKey + "/" + "transitions".
    
    postRequest (cUrl, transitionPayload).  
      
    FINALLY:
      DELETE OBJECT transitionPayload NO-ERROR.
      DELETE OBJECT transitionFields  NO-ERROR.
    END FINALLY.
    
  END METHOD.

END CLASS.
