/*------------------------------------------------------------------------
  File : rtb/jira/IssueTransitionQuery.cls
  Desc : Jira Transitions Query
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING OpenEdge.Net.HTTP.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.IssueTransitionQuery INHERITS JiraData:
  
  {rtb/jira/i/ttIssueTransition.i &reference-only=REFERENCE-ONLY}
  
  CONSTRUCTOR IssueTransitionQuery ():
    SUPER().
  END CONSTRUCTOR.
  
  METHOD PUBLIC VOID fetchTransitions (INPUT pcIssueKey AS CHARACTER, INPUT TABLE ttIssueTransition):
    
    DEFINE VARIABLE transitionData AS JsonObject    NO-UNDO.
    DEFINE VARIABLE transition     AS JsonObject    NO-UNDO.
    DEFINE VARIABLE transitions    AS JsonArray     NO-UNDO.
    
    DEFINE VARIABLE cGet AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iCnt AS INTEGER   NO-UNDO.
    DEFINE VARIABLE i    AS INTEGER   NO-UNDO.
    
    cGet = jiraConfig:jiraUrl + "issue/" + pcIssueKey + "/transitions".
    
    getRequest(cGet).
    
    IF httpResponse:StatusCode <> INTEGER(StatusCodeEnum:OK) THEN
      UNDO, THROW NEW AppError ("Could not fetch Issue Transitions.", httpResponse:StatusCode).
          
    IF TYPE-OF(httpResponse:Entity, JsonObject) THEN 
      transitionData = CAST(httpResponse:Entity, JsonObject).
      
    ASSIGN 
      transitions = transitionData:GetJsonArray("transitions")
      iCnt        = transitions:Length.
 
    DO i = 1 TO iCnt:
       
      transition = transitions:GetJsonObject(i).
      
      CREATE ttIssueTransition.
      ASSIGN 
        ttIssueTransition.transitionId   = transition:GetCharacter("id")
        ttIssueTransition.transitionName = transition:GetCharacter("name").
      
    END. 

    RETURN.
    
    FINALLY:
      /*
       I don't necessarily trust garbage collection
      */
      DELETE OBJECT transitionData NO-ERROR.
      DELETE OBJECT transitions    NO-ERROR.
      DELETE OBJECT transition     NO-ERROR.
    END FINALLY.
    
  END METHOD.

END CLASS.
