/*------------------------------------------------------------------------
  File : rtb/jira/IssueTypeQuery.cls
  Desc : Jira Issue Types Query
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING OpenEdge.Net.HTTP.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.IssueTypeQuery INHERITS JiraData:
  
  {rtb/jira/i/ttIssueType.i &reference-only=REFERENCE-ONLY}
  
  CONSTRUCTOR IssueTypeQuery ():
    SUPER().
  END CONSTRUCTOR.
  
  METHOD PUBLIC VOID fetchIssueTypes (INPUT TABLE ttIssueType):
    
    DEFINE VARIABLE issueType  AS JsonObject NO-UNDO.
    DEFINE VARIABLE issueTypes AS JsonArray  NO-UNDO.
    
    DEFINE VARIABLE cGet AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iCnt AS INTEGER   NO-UNDO.
    DEFINE VARIABLE i    AS INTEGER   NO-UNDO.
    
    cGet = jiraConfig:jiraUrl + "issuetype".
    
    getRequest(cGet).
      
    IF httpResponse:StatusCode <> INTEGER(StatusCodeEnum:OK) THEN
      UNDO, THROW NEW AppError ("Could not fetch Issue Types.", httpResponse:StatusCode).
          
    IF TYPE-OF(httpResponse:Entity, JsonObject) THEN 
      ASSIGN 
        issueType = CAST(httpResponse:Entity, JsonObject)
        iCnt    = 1.
    ELSE 
      ASSIGN 
        issueTypes = CAST(httpResponse:Entity, JsonArray)
        iCnt       = issueTypes:LENGTH. 
    
    DO i = 1 TO iCnt:
    
      IF iCnt > 1 THEN 
        issueType = issueTypes:GetJsonObject(i).
      
      IF NOT CAN-FIND(ttIssueType WHERE ttIssueType.issueTypeName = issueType:GetCharacter("name")) THEN DO:
        CREATE ttIssueType.
        ASSIGN 
          ttIssueType.issueTypeId          = issueType:GetCharacter("id")
          ttIssueType.issueTypeName        = issueType:GetCharacter("name")
          ttIssueType.issueTypeDescription = issueType:GetCharacter("description").
      END.
      
    END. 

    RETURN.
    
    FINALLY:
      /*
       I don't necessarily trust garbage collection
      */
      DELETE OBJECT issueType    NO-ERROR.
      DELETE OBJECT issueTypes   NO-ERROR.
    END FINALLY.    
    
  END METHOD.

END CLASS.
