/*------------------------------------------------------------------------
  File : rtb/jira/JiraData.cls
  Desc : Parent class for any subclass that will get Jira data.
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING OpenEdge.Net.HTTP.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.JiraData:
  
  DEFINE PROTECTED VARIABLE jiraConfig         AS JiraConfig     NO-UNDO.
  DEFINE PROTECTED VARIABLE httpClientBuilder  AS ClientBuilder  NO-UNDO.
  DEFINE PROTECTED VARIABLE httpRequestBuilder AS RequestBuilder NO-UNDO.
  DEFINE PROTECTED VARIABLE httpResponse       AS IHttpResponse  NO-UNDO.
  DEFINE PROTECTED VARIABLE httpClient         AS IHttpClient    NO-UNDO.
  
  CONSTRUCTOR JiraData ():
    THIS-OBJECT:jiraConfig = NEW JiraConfig().
  END CONSTRUCTOR.
  
  DESTRUCTOR JiraData ():
    DELETE OBJECT jiraConfig NO-ERROR.
  END DESTRUCTOR.
  
  METHOD PROTECTED VOID getRequest (INPUT pcGet AS CHARACTER):

    httpClient = ClientBuilder:Build():Client.

    httpRequestBuilder = RequestBuilder:Get(pcGet).
    httpRequestBuilder:ContentType('application/json').
    httpRequestBuilder:UsingBasicAuthentication(jiraConfig:getCredentials()).
    
    httpResponse = httpClient:Execute(httpRequestBuilder:Request).
    
  END METHOD.
  
  METHOD PROTECTED VOID postRequest (INPUT pcUrl AS CHARACTER, INPUT payLoad AS JsonObject):

    httpClient = ClientBuilder:Build():SetRequestTimeout(1):Client. /* set the timeout to work around issue */
    
    /*
     Note: The 'Origin' header is added because in our testing, the Jira server rejected the request without it
    */
    httpRequestBuilder = RequestBuilder:Post(pcUrl,payload).
                                
    httpRequestBuilder:AddHeader("Origin", jiraConfig:jiraBaseUrl).
    httpRequestBuilder:ContentType('application/json').
    httpRequestBuilder:UsingBasicAuthentication(jiraConfig:getCredentials()).
                                
    /*
     Note: This can throw an error even if successful. PSC logged it as a defect.
    */
    httpResponse = httpClient:Execute(httpRequestBuilder:REQUEST).
  
  END METHOD.

END CLASS.
