/*------------------------------------------------------------------------
  File : rtb/jira/ProjectQuery.cls
  Desc : Jira Projects Query
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING OpenEdge.Net.HTTP.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.ProjectQuery INHERITS JiraData:
  
  {rtb/jira/i/ttProject.i &reference-only=REFERENCE-ONLY}
  
  CONSTRUCTOR ProjectQuery ():
    SUPER().
  END CONSTRUCTOR.
  
  METHOD PUBLIC VOID fetchProjects (INPUT TABLE ttProject):
    
    DEFINE VARIABLE project      AS JsonObject    NO-UNDO.
    DEFINE VARIABLE projects     AS JsonArray     NO-UNDO.
    
    DEFINE VARIABLE cGet AS CHARACTER NO-UNDO.
    DEFINE VARIABLE iCnt AS INTEGER   NO-UNDO.
    DEFINE VARIABLE i    AS INTEGER   NO-UNDO.
    
    cGet = jiraConfig:jiraUrl + "project".
    
    getRequest(cGet).
      
    IF httpResponse:StatusCode <> INTEGER(StatusCodeEnum:OK) THEN
      UNDO, THROW NEW AppError ("Could not fetch Projects.", httpResponse:StatusCode).
          
    IF TYPE-OF(httpResponse:Entity, JsonObject) THEN 
      ASSIGN 
        project = CAST(httpResponse:Entity, JsonObject)
        iCnt    = 1.
    ELSE 
      ASSIGN 
        projects = CAST(httpResponse:Entity, JsonArray)
        iCnt     = projects:LENGTH. 
    
    DO i = 1 TO iCnt:
    
      IF iCnt > 1 THEN 
        project = projects:GetJsonObject(i).
      
      CREATE ttProject.
      ASSIGN 
        ttProject.projectId   = project:GetCharacter("id")
        ttProject.projectKey  = project:GetCharacter("key")
        ttProject.projectName = project:GetCharacter("name").
      
    END. 

    RETURN.
    
    FINALLY:
      /*
       I don't necessarily trust garbage collection
      */
      DELETE OBJECT project      NO-ERROR.
      DELETE OBJECT projects     NO-ERROR.
    END FINALLY.    
    
  END METHOD.

END CLASS.
