/* schupdp2.p -- Update the Progress Schema Phase 2 

This procedure runs the post-update procedures 
specific to each file schmema to transfer the data that was 
dumped prior to the schema update in schupd.p.

This proceedure produces an error file named rtb_idat/schupd.err.
*/

/* --- Define New Shared --- */
DEFINE NEW SHARED STREAM Serr.
DEFINE NEW SHARED STREAM Supdlog.

/* --- Define Locals --- */
DEFINE STREAM Sin1.
DEFINE STREAM Sout1.

DEFINE VARIABLE Mfilepos  AS INTEGER            NO-UNDO.
DEFINE VARIABLE Mlastpos  AS INTEGER            NO-UNDO.
DEFINE VARIABLE Merror    AS CHARACTER          NO-UNDO.
DEFINE VARIABLE Mctrlfile AS CHARACTER          NO-UNDO.
DEFINE VARIABLE Mtoken    AS CHARACTER          NO-UNDO.
DEFINE VARIABLE MisVer11  AS LOGICAL            NO-UNDO.
DEFINE VARIABLE Mschctrl  AS CHARACTER EXTENT 6 NO-UNDO.
   /* --- Mschctrl[1] updstatus 
          Mschctrl[2] dbase    
          Mschctrl[3] filename
          Mschctrl[4] deact-idx
          Mschctrl[5] prog-code
          Mschctrl[6] dbdir     --- */

/* --- End of defines --- */


/* --- Initialize variables. --- */
ASSIGN
   Mctrlfile = "rtb_idat/schctrl".

/* --- Create a lock file that the calling process will wait for. --- */
OUTPUT STREAM Sout1 TO rtb_idat/runlock UNBUFFERED NO-ECHO.
OUTPUT STREAM Sout1 CLOSE.

/* --- Remove lock file created by calling process. --- */
IF SEARCH("rtb_idat/strtlock") <> ? THEN 
   OS-DELETE VALUE(SEARCH("rtb_idat/strtlock")).


/* --- Create a log of all errors that occurred during schema update. --- */
OUTPUT STREAM Serr TO rtb_ilog/schupdp2.err NO-ECHO.

/* --- Create a log of all file actions perfromed during post schema
       update. --- */
OUTPUT STREAM Supdlog TO rtb_ilog/schupdp2.log UNBUFFERED NO-ECHO.

/* --- Is this an RTB 11 (or newer) deployment? --- */
MisVer11 = (SEARCH("rtb_idat/deployxi") <> ?).

IF MisVer11 THEN
  RUN VALUE(SEARCH("rtb_inst/schupdpp.p")) (INPUT FALSE). /* post-update */
ELSE DO:

  /* --- Expansion check and or processing --- */
  IF SEARCH("dbcopies") <> ? THEN DO:  
     INPUT STREAM Sin1 FROM VALUE(Mctrlfile) NO-ECHO.
     IMPORT STREAM Sin1 Mtoken.
     Mfilepos = SEEK(Sin1).
     INPUT STREAM Sin1 CLOSE.
  END.  /* if search("dbcopies") <> ? */
  ELSE
     Mfilepos = 0.
  
  /* --- Read through the rtb_idat/schctrl file and process tables in that
         file. --- */
  REPEAT:
     ASSIGN
        Mschctrl = "".
     INPUT STREAM Sin1 FROM VALUE(Mctrlfile) NO-ECHO.
     SEEK STREAM Sin1 TO Mfilepos.
     IMPORT STREAM Sin1 Mschctrl.
     ASSIGN
        Mlastpos = Mfilepos
        Mfilepos = SEEK(Sin1).
     INPUT STREAM Sin1 CLOSE.
  
     IF Mschctrl[1] = "EXPANDED" THEN NEXT.
  
     IF Mschctrl[1] = "YYY" THEN NEXT.
  
     IF Mschctrl[6] = "" THEN
        ASSIGN
           Mschctrl[6] = "rtb_idat/tempdata".
  
     IF NOT CONNECTED(Mschctrl[2]) THEN DO:
        PUT STREAM Serr UNFORMATTED
           "Database " Mschctrl[2] " not connected" SKIP.
        IF SEARCH("rtb_idat/runlock") <> ? THEN 
           OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
        RETURN.
     END.  /* if not connected... */
  
     /* Assign DICTDB alias to current database */
     IF LDBNAME("DICTDB") <> Mschctrl[2] THEN DO:
        DELETE ALIAS "DICTDB".
        CREATE ALIAS DICTDB FOR DATABASE VALUE(Mschctrl[2]).
     END.
  
     OUTPUT STREAM Sout1 TO VALUE(Mctrlfile) NO-ECHO APPEND.
  
     ASSIGN
        Merror = "".
        
     /* --- Post update processing --- */
     IF Mschctrl[1] = "YYN" THEN DO ON ERROR UNDO, LEAVE:
        ASSIGN
           Merror = "Post-update Error".
        PUT STREAM Supdlog UNFORMATTED
           "Post-Schema Processing for " + Mschctrl[3] + "..." SKIP.
        RUN VALUE("rtb_idat/" + Mschctrl[5] + "f.p")
            (Mschctrl[6]). /* directory containing data */
        SEEK STREAM Sout1 TO Mlastpos + 3.
        PUT STREAM Sout1 UNFORMATTED "Y".
        ASSIGN
           Merror = "".
     END.  /* if mschctrl[1] = "yyn" */
     
     IF Merror <> "" THEN DO:
        PUT STREAM Serr UNFORMATTED
           "An error occurred during post-schema update processing for: " +
           Mschctrl[3] SKIP.
        IF SEARCH("rtb_idat/runlock") <> ? THEN 
           OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
        RETURN.
     END.  /* if merror <> "" */
  
     OUTPUT STREAM Sout1 CLOSE.
  
  END. /* repeat */

END. /* not MisVer11 */


/* --- Close log files. --- */
OUTPUT STREAM Serr CLOSE.
OUTPUT STREAM Supdlog CLOSE.


/* --- Remove lock file so calling process can continue. --- */
IF SEARCH("rtb_idat/runlock") <> ? THEN 
   OS-DELETE VALUE(SEARCH("rtb_idat/runlock")).
